#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include "iolib.h"
const char *progname = "softlimit";
#include "freedt.h"

void set(int resource, char *value) {
	struct rlimit lim;

	if (getrlimit(resource, &lim) < 0)
		die("failed to get limit");

	if (strcmp(value, "=") == 0) {
		lim.rlim_cur = lim.rlim_max;
	} else {
		lim.rlim_cur = atoi(value);
	}

	if (setrlimit(resource, &lim) < 0)
		die("failed to set limit");
}

int main(int argc, char **argv) {
	istdinit();

	while (1) {
		/* The + is here so that GNU getopt doesn't attempt
		   to parse options of programs we execute. */
		char c = getopt(argc, argv, "+m:d:s:l:a:o:p:f:c:r:t:");
		if (c == -1)
			break;
		switch (c) {
		case 'm':
			set(RLIMIT_DATA, optarg);
			set(RLIMIT_STACK, optarg);
			set(RLIMIT_MEMLOCK, optarg);
			set(RLIMIT_AS, optarg);
			break;
		case 'd':
			set(RLIMIT_DATA, optarg);
			break;
		case 's':
			set(RLIMIT_STACK, optarg);
			break;
		case 'l':
			set(RLIMIT_MEMLOCK, optarg);
			break;
		case 'a':
			set(RLIMIT_AS, optarg);
			break;
		case 'o':
			set(RLIMIT_NOFILE, optarg);
			break;
		case 'p':
			set(RLIMIT_NPROC, optarg);
			break;
		case 'f':
			set(RLIMIT_FSIZE, optarg);
			break;
		case 'c':
			set(RLIMIT_CORE, optarg);
			break;
		case 'r':
			set(RLIMIT_RSS, optarg);
			break;
		case 't':
			set(RLIMIT_CPU, optarg);
			break;
		default:
			die("unknown option");
		}
	}

	if ((argc - optind) < 1)
		die("usage: softlimit [options] command ...");

	execvp(argv[optind], &argv[optind]);
	die2(argv[optind], "unable to exec");

	return 0; /* NOTREACHED */
}

